/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"

// This constructor sets the default properties for the OpenLDev window, creates
// the settings file if it does not exist and loads it if it does. Editor
// properties are then set & the last opened file is loaded if applicable.
OpenLDev::OpenLDev (GtkWidget *mainwin)
{  
  // Set properties of main window
  window = mainwin;
  string title = "OpenLDev " + string(VERSION);
  gtk_window_set_title (GTK_WINDOW(window), title.c_str());
  gtk_container_set_border_width (GTK_CONTAINER(window), 0);
  
  // Create settings objects
  project = new ProjectSettings();
  env = new EnvironmentSettings();
  editor = new EditorProperties();
  ptemplate = new ProjectTemplate();
  list_of_templates = new TemplateList();
  
  // Make directories & files if they do not exist
  string dir = g_get_home_dir();
  dir += "/.openldev/";
  mkdir(dir.c_str(), 0777);
  
  dir += "openldev.settings";
  if (!g_file_test(dir.c_str(), G_FILE_TEST_EXISTS))
    env->create_new_settings_file();
  else
    env->load_settings_file();
  
  // Call functions to add main window widgets & show them all
  gtk_widget_set_size_request (window, env->get_width(), env->get_height());
  editor->set_toolbar(true);
  editor->set_fileviewer(true);
  editor->set_statusbar(true);
  editor->set_buildbox(true);
  openldev_initialize_objects();
  gtk_notebook_set_current_page(GTK_NOTEBOOK(output->get_notebook()), 0);
  list_of_templates->load_templates();
  
  // Load the last project or just set the directory to the home directory
  if (env->get_lastproject() != "EMPTY" && env->get_autoload())
    openldev_open_project(env->get_lastproject());
}

// Create all of the objects needed by the main openldev window. Then call the
// functions to populate the find toolbar and goto line tolbar. Lastly, connect
// the signals to their widgets.
void OpenLDev::openldev_initialize_objects()
{
  vpaned = gtk_vpaned_new();
  hpaned = gtk_hpaned_new();
  vbox = gtk_vbox_new(false, 0);
  hbox = gtk_hbox_new(false, 0);
  status_bar = gtk_statusbar_new();
  status_bar2 = gtk_statusbar_new();
  output = new OutputManager();
  noteman = new NotebookManager();
  browser = new Browser();
  
  // Add all of the widgets & show them
  openldev_create_find_toolbar();
  openldev_create_goto_line_toolbar();
  openldev_populate_window();
  gtk_widget_show_all (window);
  findtoolbar->show_error(false);
  gototoolbar->show_error(false);
  on_close_toolbars (findtoolbar->get_hbox(), (gpointer)this);
  openldev_connect_signals();
}

// Set the properties of all of the widgets found in the main openldev window.
// This includes loading the menu, toolbar, project files TreeView, notebook
// manager, build output ListStore and the status bar
void OpenLDev::openldev_populate_window()
{
  openldev_create_menus_and_toolbar();
  openldev_initialize_treeview();
  
  // Add menu & toolbar to main vbox
  gtk_box_pack_start(GTK_BOX(vbox), menubar, false, false, 0);
  gtk_box_pack_start(GTK_BOX(vbox), toolbar, false, false, 0);
  gtk_paned_pack1(GTK_PANED(hpaned), browser->get_notebook(), false, true);
  gtk_paned_pack2(GTK_PANED(hpaned), noteman->noteman_get_notebook(), true, true);
  gtk_paned_pack1(GTK_PANED(vpaned), hpaned, true, true);
  gtk_paned_pack2(GTK_PANED(vpaned), output->get_notebook(), false, true);

  // Status bar handling
  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR(status_bar), false);
  gtk_widget_set_size_request (status_bar, env->get_treeview_width(), 1);
  gtk_box_pack_start(GTK_BOX(hbox), status_bar, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hbox), status_bar2, true, true, 0);
  gtk_box_set_spacing(GTK_BOX(hbox), 5);

  // Add scrolled_win3 to the second vbox and finish off with the statusbar
  gtk_box_pack_start(GTK_BOX(vbox), vpaned, true, true, 0);
  gtk_box_pack_start(GTK_BOX(vbox), gototoolbar->get_hbox(), false, false, 0);
  gtk_box_pack_start(GTK_BOX(vbox), findtoolbar->get_hbox(), false, false, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, false, false, 0);

  // Add v' & h' boxes to the main window
  gtk_container_add (GTK_CONTAINER(window), vbox);
}

void OpenLDev::openldev_connect_signals()
{
  g_signal_connect (G_OBJECT(window), "event-after",
                    G_CALLBACK(on_resize), (gpointer)this);
  g_signal_connect (G_OBJECT(hpaned), "event-after",
                    G_CALLBACK(on_resize_pane), (gpointer)this);
  g_signal_connect (G_OBJECT(vpaned), "event-after",
                    G_CALLBACK(on_resize_pane), (gpointer)this);
  g_signal_connect (G_OBJECT(browser->get_files()), "button-press-event",
                    G_CALLBACK(on_button_press_event_treeview), (gpointer)this);
  g_signal_connect (G_OBJECT(output->get_search_box()), "button-press-event", 
                    G_CALLBACK(on_search_box_selected), (gpointer)this);
  g_signal_connect (G_OBJECT(output->get_build_box()), "button-press-event",
                    G_CALLBACK(on_build_box_selected), (gpointer)this);
}

// Quit the program
void openldev_quit (GtkMenuItem* menuitem, 
                    gpointer data) 
{ 
  OpenLDev *ptr = (OpenLDev*) data;
  gtk_widget_hide(ptr->window);
}

// When the window is resized, this checks the width and height and stores
// them so the same size can be restored on restart.
void on_resize (GtkWidget *widget,
                GdkEvent *event,
                gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (event->type == GDK_CONFIGURE)
  {
    // Get the current height & width and store them
    int width=0, height=0;
    width = ptr->window->allocation.width;
    height = ptr->window->allocation.height;
    ptr->env->set_width(width);
    ptr->env->set_height(height);
    ptr->env->write_settings_file();
  }
}

// When one of the panes is resized, this checks the width and height of each
// pane so the same sizes can be restored on restart.
void on_resize_pane (GtkWidget *widget,
                     GdkEvent *event,
                     gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (event->type == GDK_MOTION_NOTIFY)
  {   
    // Get the Project File & Output panes width/height & store them
    ptr->env->set_treeview_width(ptr->browser->get_files_swin()->allocation.width);
    ptr->env->set_buildbox_height(ptr->output->get_build_swin()->allocation.height);
    ptr->env->write_settings_file();
    gtk_widget_set_size_request (ptr->status_bar, ptr->env->get_treeview_width(), 1);
  }
}

// Close project file. First call the close project funtion and then erase
// the project files treeview, save all of the files and erase all of the tabs.
void openldev_close_project (GtkMenuItem* menuitem, 
                             gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  ptr->project->close_project();
  ptr->openldev_populate_treeview();
  ptr->noteman->noteman_save_all_files();
  ptr->noteman->noteman_erase_all_tabs();
}

// Update the status bar. This function receives the iterator of the cursor
// position and the mark at that point. It then calculates the line and column
// position and adds them with the project name an version to the status bar.
// The project and version are labelled as 'N/A' if a project is not open.
void on_status_bar_update (GtkTextBuffer *buffer,
                           GtkTextIter *insert, 
                           GtkTextMark *mark,
                           gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  string text;
  char vertical[5], horizontal[5];
  g_snprintf(vertical, sizeof(vertical), "%d", gtk_text_iter_get_line(insert)+1);
  g_snprintf(horizontal, sizeof(horizontal), "%d", gtk_text_iter_get_line_offset(insert)+1);
       
  if (!ptr->project->get_name().empty())
    text = string(" Project: ") + ptr->project->get_name() 
                + "       Version: " + ptr->project->get_version() 
                + "       Line: " + vertical 
                + "       Column: " + horizontal;
  else
    text = string(" Project: None\tVersion: N/A\tLine: ") 
                + vertical + "       Column: " + horizontal;
  
  gtk_statusbar_push(GTK_STATUSBAR(ptr->status_bar2), 0, text.c_str());
}

bool on_textview_handle_autocomplete (GtkWidget *widget,
                                      GdkEventKey *event,
                                      gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (event->type == GDK_KEY_PRESS)
  {
    gint x, y;
    gtk_window_set_gravity (GTK_WINDOW(ptr->window), GDK_GRAVITY_NORTH_WEST);
    gtk_window_get_position (GTK_WINDOW(ptr->window), &x, &y);    
    autocomplete_position (data, x, y);
  }
  
  return false;
}
