/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "project_properties.h"

// The project properties dialog initializes the object, set their properties,
// creates the dialog & shows it. It receives the project properties object.
ProjectPropertiesDialog::ProjectPropertiesDialog (ProjectSettings* project)
{
  // Create Makefile.am location string
  makefile_location = project->get_location() + "src/Makefile.am";
  initialize_objects();
  
  // Create entries for project properties
  gtk_entry_set_text(GTK_ENTRY(name), project->get_name().c_str());
  gtk_entry_set_text(GTK_ENTRY(author), project->get_author().c_str());
  gtk_entry_set_text(GTK_ENTRY(email), project->get_email().c_str());
  gtk_entry_set_text(GTK_ENTRY(homepage), project->get_url().c_str());
  gtk_entry_set_text(GTK_ENTRY(version), project->get_version().c_str());
  
  // Place the add button & compiler commands entry in an hbox together
  g_signal_connect (G_OBJECT(add), "clicked",
                    G_CALLBACK(on_add), (gpointer)this);
  
  // Populate the command_list ListStore and load the commands from the Makefile.am
  populate_liststore();
  load_compiler_commands_from_makefile();
  gtk_window_set_default(GTK_WINDOW(dialog), ok);
}

GtkWidget* ProjectPropertiesDialog::get_dialog()
{ return dialog;
}

// Initialize all of the widgest for the project information dialog.
void ProjectPropertiesDialog::initialize_objects()
{
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/projectproperties.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);
  
  dialog = glade_xml_get_widget(xml, "dialog");
  cancel = glade_xml_get_widget(xml, "button_cancel");
  ok = glade_xml_get_widget(xml, "button_ok");
  add = glade_xml_get_widget(xml, "button_add");
  name = glade_xml_get_widget(xml, "name");
  author = glade_xml_get_widget(xml, "author");
  email = glade_xml_get_widget(xml, "email");
  homepage = glade_xml_get_widget(xml, "homepage");
  version = glade_xml_get_widget(xml, "version");
  compiler = glade_xml_get_widget(xml, "commands");
  command_list = glade_xml_get_widget(xml, "treeview");
}

// Populate the command_list ListStore with all of the types of compiler
// commands that are available by default and their descriptions. The user
// can add their own by adding it directly to the GtkEntry
void ProjectPropertiesDialog::populate_liststore()
{
  GtkTreeModel *model;
  GtkCellRenderer *cell_renderer;
  GtkTreeViewColumn *column;
        
  model = GTK_TREE_MODEL(gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING));
        
  cell_renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Command", 
                          cell_renderer, "text", 0, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(command_list), column);
  
  cell_renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Description", 
                          cell_renderer, "text", 1, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(command_list), column);
	
  // Add the rows to the ListStore
  add_row(model, "-g", "Produce debugging information in native format");
  add_row(model, "-Idir", "Add a directory to search for header files");
  add_row(model, "-O0", "Do not optimize at all");
  add_row(model, "-O1", "Reduce code size & execution time");
  add_row(model, "-O2", "Optimize without sacrificing speed");
  add_row(model, "-O3", "High level optimization");
  add_row(model, "-Os", "Optimize only to reduce code size");
  add_row(model, "-w", "Suppress all warnings");
  add_row(model, "-Wall", "Turn on all standard warning messages");
    
  gtk_tree_view_set_model(GTK_TREE_VIEW(command_list), model);
	g_object_unref(model);
}

// Add a row to the ListStore model. This function receives the command name
// and its description.
void ProjectPropertiesDialog::add_row (GtkTreeModel *model,
                                       string comm, 
                                       string descrip)
{
  GtkTreeIter iter;
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, comm.c_str(), 1, descrip.c_str(), -1);
}

// Load the compiler commands from the Makefile.am
void ProjectPropertiesDialog::load_compiler_commands_from_makefile()
{
  string text;
  gint location;
  
  text = read_file(makefile_location);
  
  // Simplify to get out the compiler flags & remove returns and \'s
  location = text.find("FLAGS =", 0);
  text.erase(0, location+7);
  while (text.find("\n", 0) != string::npos)
    text.erase(text.find("\n", 0), 1);
  
  while (text.find("\\", 0) != string::npos)
    text.erase(text.find("\\", 0), 1);
  
  location = text.find("bin_PROGRAMS", 0);
  text.erase(location, text.length()-location);
  
  // Simplify white space
  location = text.find("-", 0);
  text.erase(0, location);
  while (text.find("  ", 0) != string::npos)
    text.erase(text.find("  ", 0), 1);
  
  while (text.find("\t", 0) != string::npos)
    text.erase(text.find("\t", 0), 1);
  
  gtk_entry_set_text(GTK_ENTRY(compiler), text.c_str());
}

// Add the selected compiler command to the compiler entry. The function
// receives the command string. It makes sure the -Wall and -w commands
// are not shown at the same time and that -Idir is replaced by
// -I/usr/include/directory
void ProjectPropertiesDialog::add_compiler_command (string c)
{
  string text = gtk_entry_get_text(GTK_ENTRY(compiler));
  int location = text.find(c, 0);
  
  // Check if it already exists
  if (location < 0)
  {
    // Add optimization command (Remove others)
    if (c.find("-O", 0) != string::npos)
    {
      if (text.find("-O", 0) != string::npos)
        text.erase(text.find("-O", 0), 4);
    }
      
    // Add the directory to the include command
    else if (c == "-Idir")
      c = "-I/usr/include/directory";
    
    // Make sure warning commands do not conflict
    else if (c == "-Wall" && text.find("-w", 0) != string::npos)
      text.erase(text.find("-w", 0), 3);
    else if (c == "-w" && text.find("-Wall", 0) != string::npos)
      text.erase(text.find("-Wall", 0), 6);
  
    text += " " + c;
    gtk_entry_set_text(GTK_ENTRY(compiler), text.c_str());
  }
}

// Check which command is selected by getting it from the TreeView. Call the
// function to add it to the compiler commands.
void on_add (GtkWidget *widget,
             gpointer data)
{
  ProjectPropertiesDialog *ptr = (ProjectPropertiesDialog*) data;
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ptr->command_list));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
  {
    gchar *text;
    gtk_tree_model_get(model, &iter, 0, &text, -1);
    ptr->add_compiler_command(text);
  }
}

// Get private project properties and give the main openldev class access to them
string ProjectPropertiesDialog::get_name()
{ return gtk_entry_get_text(GTK_ENTRY(name)); 
}

string ProjectPropertiesDialog::get_author()
{ return gtk_entry_get_text(GTK_ENTRY(author)); 
}

string ProjectPropertiesDialog::get_email()
{ return gtk_entry_get_text(GTK_ENTRY(email)); 
}

string ProjectPropertiesDialog::get_homepage()
{ return gtk_entry_get_text(GTK_ENTRY(homepage)); 
}

string ProjectPropertiesDialog::get_version()
{ return gtk_entry_get_text(GTK_ENTRY(version)); 
}

string ProjectPropertiesDialog::get_commands()
{ return gtk_entry_get_text(GTK_ENTRY(compiler)); 
}
