/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "utils.h"

// Throw an error dialog with the specified message
void error (string message)
{
  GtkWidget *dialog;
  dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, 
                                   GTK_BUTTONS_CLOSE, message.c_str());
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  
  switch (result)
  {
    default:
      gtk_widget_hide(dialog);
  }
}

// Throw a warning dialog with the specified message
void warning (string message)
{
  GtkWidget *dialog;
  dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, 
                                   GTK_BUTTONS_CLOSE, message.c_str());
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  
  switch (result)
  {
    default:
      gtk_widget_hide(dialog);
  }
}

// Show a question dialog with the specified message. Return the answer the
// user choses. This function handles creating the dialog, setting the properties
// and then deleting it and returning the answer.
bool question (string question)
{
  GtkWidget *dialog = gtk_dialog_new(),
            *yes = gtk_button_new_from_stock("gtk-yes"),
            *no = gtk_button_new_from_stock("gtk-no"),
            *image = gtk_image_new_from_stock("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG),
            *label = gtk_label_new(question.c_str()),
            *hbox = gtk_hbox_new(false, 10);
  
  gtk_container_set_border_width (GTK_CONTAINER(hbox), 10);
  gtk_box_pack_start (GTK_BOX(hbox), image, false, false, 0);
  gtk_box_pack_start (GTK_BOX(hbox), label, false, false, 0);
  
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
                      hbox, TRUE, TRUE, 0);
  gtk_box_set_spacing (GTK_BOX(GTK_DIALOG(dialog)->vbox), 7);
  gtk_window_set_title (GTK_WINDOW(dialog), "Question");  
  gtk_window_set_modal (GTK_WINDOW(dialog), true);
  gtk_window_set_resizable (GTK_WINDOW(dialog), false);
  gtk_dialog_set_has_separator (GTK_DIALOG(dialog), true);
  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), no, -9);
  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), yes, -8);
  
  gtk_widget_show_all(dialog);
  
  switch (gtk_dialog_run(GTK_DIALOG(dialog)))
  {
    case -8:
    {
      gtk_widget_hide(dialog);
      return true;
      break;
    }
    default:
    {
      gtk_widget_hide(dialog);
      return false;
      break;
    }
  }
}

// Set the tab width for when the file contains tabs instead of spaces. This
// function receives the font description, environment settings object and the
// current textview. It returns the tabarray from the textview
PangoTabArray* get_tab_array(PangoFontDescription *fd, 
                             EnvironmentSettings *env,
                             GtkWidget *text_view)
{
  PangoTabArray *tab_array;
  PangoLayout *layout;
	string tab;
	int i=0, width, height;

  // Add spaces for the number of spaces per tab
	while (i < env->get_tabsize()) 
  {
		tab += " ";
		i++;
	}

  // Create the pango layout & get the pixel width
  layout = gtk_widget_create_pango_layout(text_view, tab.c_str());
  pango_layout_set_font_description(layout, fd);
  pango_layout_get_pixel_size(layout, &width, &height);
 
  // Create the tab array from the width
  tab_array = pango_tab_array_new(1, true);
  pango_tab_array_set_tab(tab_array, 0, PANGO_TAB_LEFT, width);

  return tab_array;
}

// Create new stock menu item. This function receives the icon factory, the
// name of the item, the id of the item, and the filename of the icon.
void add_new_stock_item (GtkIconFactory *icon_factory, 
                         string name,
                         string id, 
                         string filename)
{
  GtkIconSource *icon_source = gtk_icon_source_new();
  GtkIconSet *icon = gtk_icon_set_new();
  
  // Set the properties of the stock menu/toolbar item
  gtk_icon_source_set_icon_name (icon_source, name.c_str());
  gtk_icon_source_set_filename (icon_source, filename.c_str());
  gtk_icon_set_add_source (icon, icon_source);
  gtk_icon_factory_add (icon_factory, id.c_str(), icon);
}

// Read in the text from a file. This function receives a filename and returns
// the text string of the file.
string read_file(string filename)
{
  if (!g_file_test(filename.c_str(), G_FILE_TEST_EXISTS))
    return string("");
  
  string text;
  string line;
  ifstream fin(filename.c_str());
  while (getline(fin, line))
    text += g_locale_to_utf8 (line.c_str(), -1, NULL, NULL, NULL) + string("\n");
  
  return text;
}

// Write the text to the file. This function receives the filename and the file
// text. The text is then written to the file located at the filename.
void write_file (string filename, 
                 string text)
{
  ofstream fout(filename.c_str());
  fout << text;
  fout.close();
}

// Get file name and line of the error or warning. This function parses the
// error or warning sent by the output string. Output is then converted into
// only the short filename containing the error & the line number of the
// error or warning is returned.
int get_file_name_and_line (string &output)
{
  string line;
  int location = output.find(":", 0);
  
  // Get the line number & delete it out so only filename is left
  line = output.substr(location+1, output.length()-location-1);
  output.erase(location, output.length()-location);
  
  // Erase all except for the line number
  location = line.find(":", 0);
  line.erase(location, line.length()-location);
  
  // Return the line number
  return atol(line.c_str());
}
